//-----------------------------------------------------------------------------
// File: particles.cpp
//
// Desc: use the simplest hlsl vertex and pixel shader code to display a particles system
//
// author : adrien.auclair
//	        adrien.auclair@club-internet.fr
// date   : 07/10/2004
//-----------------------------------------------------------------------------
#define D3D_DEBUG_INFO

#include <Windows.h>
#include <mmsystem.h>
#include <d3dx9.h>
#include <dxerr9.h>
#include <assert.h>

#include "util.h"
#include "Xparticles_system.h"

#define NB_PARTICLES 1000
#define SPEED_TRANSITOIRE 1000

#define SAFE_RELEASE(x) { if (x!=NULL) { x->Release(); x=NULL; } }

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
LPDIRECT3D9             g_pD3D                 = NULL; // Used to create the D3DDevice
LPDIRECT3DDEVICE9       g_pd3dDevice           = NULL; // Our rendering device
LPDIRECT3DVERTEXBUFFER9 g_pVB_Walls            = NULL; // Buffer to hold vertices for the wall
LPDIRECT3DVERTEXBUFFER9 g_pVB_Floor            = NULL; // Buffer to hold vertices for the floor

LPDIRECT3DTEXTURE9      g_pTexture             = NULL; // The wall texture
LPDIRECT3DTEXTURE9      g_pTextureFloor        = NULL; // The floor texture
LPDIRECT3DTEXTURE9      g_pWallsNormalMap      = NULL; // The walls normal map
LPDIRECT3DTEXTURE9      g_pFloorNormalMap      = NULL; // The floor normal map
LPDIRECT3DTEXTURE9      g_pParticuleTexture    = NULL; // The wall texture

LPDIRECT3DVERTEXSHADER9      m_pBumpVertexShader      = NULL;  // vertex shader
LPDIRECT3DVERTEXDECLARATION9 m_pBumpVertexDeclaration = NULL;  // vertex shader input data type
LPD3DXCONSTANTTABLE          m_pBumpVertexConstants   = NULL;  // array of vertex shader constants

LPDIRECT3DVERTEXSHADER9      m_pParticlesVertexShader      = NULL;  // vertex shader
LPDIRECT3DVERTEXDECLARATION9 m_pParticlesVertexDeclaration = NULL;  // vertex shader input data type
LPD3DXCONSTANTTABLE          m_pParticlesVertexConstants   = NULL;  // array of vertex shader constants

LPDIRECT3DPIXELSHADER9       m_pBumpPixelShader       = NULL;  // pixel shader for bump mapping
LPD3DXCONSTANTTABLE          m_pBumpPixelConstants    = NULL;  // array of pixel shader constant for bump mapping

LPDIRECT3DPIXELSHADER9       m_pPassPixelShader       = NULL;  // pixel shader 
LPD3DXCONSTANTTABLE          m_pPassPixelConstants    = NULL;  // array of pixel shader constant


D3DXMATRIXA16 matLightView;
D3DXMATRIXA16 matProjTexture;
D3DXMATRIXA16 matTMProjTexture;

D3DXMATRIXA16 matView;
D3DXMATRIXA16 matWorld;    
D3DXMATRIXA16 matProj;    
D3DXVECTOR3   vEyePt( 0.5f, 0.7f,-1.2f );
D3DXMATRIX  MFull,TMFull; // Full = World * View * Projection
D3DXMATRIX  MLightFull,TMLightFull; // Full = World * View * Projection

XParticles_System * particules;


D3DXVECTOR4 LightPos;                      //position of the light in world space
const D3DXVECTOR3 center(0.5f,0.9f,0.5f);  // center of the circle for the path of the light
const float rayon = 0.3f;                  // radius of this circle
float angle = 0.0f;                        // current angle
float angle_step = 0.01f;                  // step

// coeff for distance attenuation for lithing
const float kC = 0.1f;
const float kL = 0.5f;
const float kQ = 1.0f;
// coeff diffuse
float kD = 1.0f;
//coeff ambient
float kA = 0.0f;

// A structure for our custom vertex type
struct CUSTOMVERTEX
{
    FLOAT x, y, z;      // The untransformed, 3D position for the vertex	
	FLOAT nx,ny,nz;	    // the normal for ligthing (per vertex)	
	FLOAT tx,ty,tz;     // tangent
	FLOAT tu, tv;       // The texture coordinates
	FLOAT ntu, ntv;     // The texture coordinates in the normal map
};



//-----------------------------------------------------------------------------
// Name: InitBumpShaders()
// Desc: Initialize vertex and pixel shaders for bump mapping
//-----------------------------------------------------------------------------
HRESULT InitParticlesShaders()
{
	//--------------------------- creation du vertex shader
	DWORD dwFlags = 0; 
    dwFlags |= D3DXSHADER_DEBUG;
    LPD3DXBUFFER pCode   = NULL;
    LPD3DXBUFFER pErrMsg = NULL;
	HRESULT       s;

	// array describing the data type :
	D3DVERTEXELEMENT9 decl[] =
    {
      { 0,  0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },      	  	  	  
	  { 0,  12, D3DDECLTYPE_FLOAT1, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },      	  	  	  
        D3DDECL_END()
    };

	// the type of data passed to the vertex shader is created
    s = g_pd3dDevice->CreateVertexDeclaration(decl,&m_pParticlesVertexDeclaration);
    if(FAILED(s))
    {
		MessageBox(NULL,"CreateVertexDeclaration failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return false;
    }

	// compile the HLSL source
	s=D3DXCompileShaderFromFile("vs_particles.cg",NULL,NULL,"main","vs_2_0",dwFlags,&pCode,&pErrMsg,&m_pParticlesVertexConstants);
	if( FAILED(s))
	{
		if (pErrMsg != NULL) MessageBox(NULL, (char *)(pErrMsg->GetBufferPointer()), "HLSL compilation error", MB_OK);
			else MessageBox(NULL,DXGetErrorString9(s), "HLSL compilation error", MB_OK);
        return E_FAIL;
	}

	// create the shader from the compiled code
	char* szCode = (char*)pCode->GetBufferPointer();
    s = g_pd3dDevice->CreateVertexShader((DWORD*)pCode->GetBufferPointer(),&m_pParticlesVertexShader);
    pCode->Release();
    if(FAILED(s))
    {
		MessageBox(NULL,"CreateVertexShader failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;		
    }

	//------------------ creation du pixel shader	
	// compile the source code
	
	pErrMsg = NULL;
    s = D3DXCompileShaderFromFile("ps_pass.cg",NULL,NULL,"main","ps_2_0",dwFlags,&pCode,&pErrMsg,&m_pPassPixelConstants);
	if( FAILED(s))
	{
		if (pErrMsg != NULL) MessageBox(NULL, (char *)(pErrMsg->GetBufferPointer()), "HLSL compilation error", MB_OK);
			else MessageBox(NULL,DXGetErrorString9(s), "HLSL compilation error", MB_OK);
        return E_FAIL;
	}    

	// create the pixel shader from the copiled code    
    s  = g_pd3dDevice->CreatePixelShader((DWORD*)pCode->GetBufferPointer(),&m_pPassPixelShader);
    pCode->Release();
    if(FAILED(s))
    {
		MessageBox(NULL,"CreatePixelShader failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return false;
    }	
	
	return S_OK;
}



//-----------------------------------------------------------------------------
// Name: InitBumpShaders()
// Desc: Initialize vertex and pixel shaders for bump mapping
//-----------------------------------------------------------------------------
HRESULT InitBumpShaders()
{
	//--------------------------- creation du vertex shader
	DWORD dwFlags = 0; 
    dwFlags |= D3DXSHADER_DEBUG;
    LPD3DXBUFFER pCode   = NULL;
    LPD3DXBUFFER pErrMsg = NULL;
	HRESULT       s;

	// array describing the data type :
	D3DVERTEXELEMENT9 decl[] =
    {
      { 0,  0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },      
	  { 0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },
	  { 0, 24, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },	  
	  { 0, 36, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 1 },	  
	  { 0, 44, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 2 },	  
        D3DDECL_END()
    };

	// the type of data passed to the vertex shader is created
    s = g_pd3dDevice->CreateVertexDeclaration(decl,&m_pBumpVertexDeclaration);
    if(FAILED(s))
    {
		MessageBox(NULL,"CreateVertexDeclaration failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return false;
    }

	// compile the HLSL source
	s=D3DXCompileShaderFromFile("vs_bump.cg",NULL,NULL,"main","vs_2_0",dwFlags,&pCode,&pErrMsg,&m_pBumpVertexConstants);
	if( FAILED(s))
	{
		if (pErrMsg != NULL) MessageBox(NULL, (char *)(pErrMsg->GetBufferPointer()), "HLSL compilation error", MB_OK);
			else MessageBox(NULL,DXGetErrorString9(s), "HLSL compilation error", MB_OK);
        return E_FAIL;
	}

	// create the shader from the compiled code
	char* szCode = (char*)pCode->GetBufferPointer();
    s = g_pd3dDevice->CreateVertexShader((DWORD*)pCode->GetBufferPointer(),&m_pBumpVertexShader);
    pCode->Release();
    if(FAILED(s))
    {
		MessageBox(NULL,"CreateVertexShader failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;		
    }

	//------------------ creation du pixel shader
	// compile the source code
	pErrMsg = NULL;
    s = D3DXCompileShaderFromFile("ps_bump.cg",NULL,NULL,"main","ps_2_0",dwFlags,&pCode,&pErrMsg,&m_pBumpPixelConstants);
	if( FAILED(s))
	{
		if (pErrMsg != NULL) MessageBox(NULL, (char *)(pErrMsg->GetBufferPointer()), "HLSL compilation error", MB_OK);
			else MessageBox(NULL,DXGetErrorString9(s), "HLSL compilation error", MB_OK);
        return E_FAIL;
	}    

	// create the pixel shader from the copiled code
    //szCode = (char*)pCode->GetBufferPointer();
    s  = g_pd3dDevice->CreatePixelShader((DWORD*)pCode->GetBufferPointer(),&m_pBumpPixelShader);
    pCode->Release();
    if(FAILED(s))
    {
		MessageBox(NULL,"CreatePixelShader failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return false;
    }	

	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: InitShaders()
// Desc: Initialize vertex and pixel shaders
//-----------------------------------------------------------------------------
HRESULT InitShaders()
{
	if( FAILED(InitBumpShaders() ))
		return E_FAIL;	

	if( FAILED(InitParticlesShaders() ))
		return E_FAIL;	

	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: InitD3D()
// Desc: Initializes Direct3D
//-----------------------------------------------------------------------------
HRESULT InitD3D( HWND hWnd )
{
    // Create the D3D object.
    if( NULL == ( g_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
        return E_FAIL;

    // Set up the structure used to create the D3DDevice
    D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory( &d3dpp, sizeof(d3dpp) );
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
	d3dpp.BackBufferCount = 1;
	d3dpp.hDeviceWindow = hWnd;
	d3dpp.EnableAutoDepthStencil = TRUE;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

    // Create the D3DDevice
    if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_HARDWARE_VERTEXPROCESSING,
                                      &d3dpp, &g_pd3dDevice ) ) )
    {
		MessageBox(NULL, "Could not create the device", "Textures.exe", MB_OK);
        return E_FAIL;
    }

    // Turn off culling, so we see the front and back of the triangle
    g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );

    // Turn off D3D lighting, since we are providing our own vertex colors
    g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );	

	g_pd3dDevice->SetRenderState( D3DRS_POINTSPRITEENABLE, TRUE );
	
	
	if( FAILED(InitShaders() ))
		return E_FAIL;

	InitTimer();
    if FAILED(InitFont()) 
		return E_FAIL;

	

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: InitTextures()
// Desc: Load the needed textures
//-----------------------------------------------------------------------------
HRESULT InitTextures()
{
	// Use D3DX to create a texture from a file based image
    if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, "metal.bmp", &g_pTexture ) ) )
    {
        MessageBox(NULL, "Could not find metal.bmp", "Textures.exe", MB_OK);        
		return E_FAIL;
    }

	// Use D3DX to create a texture from a file based image
    if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, "particle.bmp", &g_pParticuleTexture ) ) )
    {
        MessageBox(NULL, "Could not find particule.bmp", "Textures.exe", MB_OK);        
		return E_FAIL;
    }


	// Use D3DX to create a texture from a file based image
    if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, "floor.bmp", &g_pTextureFloor ) ) )
    {
        MessageBox(NULL, "Could not find chessboard.bmp", "Textures.exe", MB_OK);
		return E_FAIL;        
    }

	// Use D3DX to create a texture from a file based image
    if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, "normal_map.bmp", &g_pWallsNormalMap ) ) )
    {
        MessageBox(NULL, "Could not find normal_map.bmp", "Textures.exe", MB_OK);
        return E_FAIL;        
    }

	// Use D3DX to create a texture from a file based image
    if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, "normal_floor.bmp", &g_pFloorNormalMap ) ) )
    {
        MessageBox(NULL, "Could not find leaves_normal_map.bmp", "Textures.exe", MB_OK);
        return E_FAIL;        
    }

	return S_OK;
}



//-----------------------------------------------------------------------------
// Name: InitGeometry()
// Desc: Creates the scene geometry
//-----------------------------------------------------------------------------
HRESULT InitGeometry()
{
	
	if( FAILED( InitTextures() ))
		return E_FAIL;
	
	CUSTOMVERTEX g_Vertices[] =
    {   // position           normal                            tangente              texture      normal_texture
        {  0.0f, 1.0f, 0.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       0.0f , 0.0f, 0.0f , 0.0f},  //1
        {  0.0f, 0.0f, 0.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       0.0f , 1.0f, 0.0f , 1.0f},  //2        
		{  0.0f, 1.0f, 1.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       1.0f , 0.0f, 1.0f , 0.0f},  //3
		
		{  0.0f, 1.0f, 1.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       1.0f , 0.0f, 1.0f , 0.0f},  //4
        {  0.0f, 0.0f, 0.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       0.0f , 1.0f, 0.0f , 1.0f},  //5        
		{  0.0f, 0.0f, 1.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       1.0f , 1.0f, 1.0f , 1.0f},  //6

		{  0.0f, 1.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       0.0f , 0.0f, 0.0f , 0.0f},  //7		
		{  0.0f, 0.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       0.0f , 1.0f, 0.0f , 1.0f},  //8
		{  1.0f, 1.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       1.0f , 0.0f, 1.0f , 0.0f},  //9	

		{  1.0f, 1.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       1.0f , 0.0f, 1.0f , 0.0f},  //10
		{  0.0f, 0.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       0.0f , 1.0f, 0.0f , 1.0f},  //11
		{  1.0f, 0.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       1.0f , 1.0f, 1.0f , 1.0f},  //12		


		{  1.0f, 1.0f, 1.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      0.0f , 0.0f, 0.0f , 0.0f},  //13
		{  1.0f, 0.0f, 0.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      1.0f , 1.0f, 1.0f , 1.0f},  //14
		{  1.0f, 1.0f, 0.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      1.0f , 0.0f, 1.0f , 0.0f},  //15		

		{  1.0f, 0.0f, 0.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      1.0f , 1.0f, 1.0f , 1.0f},  //16
		{  1.0f, 1.0f, 1.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      0.0f , 0.0f, 0.0f , 0.0f},  //17
		{  1.0f, 0.0f, 1.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      0.0f , 1.0f, 0.0f , 1.0f},  //18		
		
    };
	
    // Create the vertex buffer.
    if( FAILED( g_pd3dDevice->CreateVertexBuffer( 18*sizeof(CUSTOMVERTEX),
                                                  0, 0,  //on ne passe du format FVF : D3DFVF_CUSTOMVERTEX
                                                  D3DPOOL_MANAGED, &g_pVB_Walls, NULL ) ) )
    {
		MessageBox(NULL, "Could not create vertex buffer", "Textures.exe", MB_OK);
        return E_FAIL;
    }

    // Fill the vertex buffer.
    VOID* pVertices;
    if( FAILED( g_pVB_Walls->Lock( 0, sizeof(g_Vertices), (void**)&pVertices, 0 ) ) )
        return E_FAIL;
    memcpy( pVertices, g_Vertices, sizeof(g_Vertices) );
    g_pVB_Walls->Unlock();


	//----------------- FLOOR	
	// format is      x,y,z      normal                  tangente           texture      normal_texture
    CUSTOMVERTEX g_Vertices2[] =
    {
        {  0.0f, 0.0f, 0.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 0.0f , 1.0f , 0.0f , 1.0f  },  //1
        {  1.0f, 0.0f, 1.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 1.0f , 0.0f , 1.0f , 0.0f  },   //2
        {  0.0f, 0.0f, 1.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 0.0f , 0.0f , 0.0f , 0.0f },   //3

		{  0.0f, 0.0f, 0.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 0.0f , 1.0f , 0.0f , 1.0f  },  //4
        {  1.0f, 0.0f, 0.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 1.0f , 1.0f , 1.0f , 1.0f },   //5
        {  1.0f, 0.0f, 1.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 1.0f , 0.0f , 1.0f , 0.0f  }    //6		
    };

    // Create the vertex buffer.
    if( FAILED( g_pd3dDevice->CreateVertexBuffer( 6*sizeof(CUSTOMVERTEX),
                                                  0, 0, // on se passe de D3DFVF_CUSTOMVERTEX,
                                                  D3DPOOL_MANAGED, &g_pVB_Floor, NULL ) ) )
    {
		MessageBox(NULL, "Could not create vertex buffer", "Textures.exe", MB_OK);
        return E_FAIL;
    }

    // Fill the vertex buffer.    
    if( FAILED( g_pVB_Floor->Lock( 0, sizeof(g_Vertices2), (void**)&pVertices, 0 ) ) )
        return E_FAIL;
    memcpy( pVertices, g_Vertices2, sizeof(g_Vertices2) );
    g_pVB_Floor->Unlock();
	

    return S_OK;
}



//-----------------------------------------------------------------------------
// Name: Cleanup()
// Desc: Releases all previously initialized objects
//-----------------------------------------------------------------------------
VOID Cleanup()
{	
	SAFE_RELEASE(g_pParticuleTexture);

	SAFE_RELEASE(g_pTexture);
	SAFE_RELEASE(g_pTextureFloor);
	
	SAFE_RELEASE(g_pWallsNormalMap);
	SAFE_RELEASE(g_pFloorNormalMap);	

	SAFE_RELEASE(m_pBumpVertexShader);		
	SAFE_RELEASE(m_pBumpVertexDeclaration);
	SAFE_RELEASE(m_pBumpVertexConstants);

	SAFE_RELEASE(m_pParticlesVertexShader);		
	SAFE_RELEASE(m_pParticlesVertexDeclaration);
	SAFE_RELEASE(m_pParticlesVertexConstants);
	
	SAFE_RELEASE(m_pBumpPixelShader);
	SAFE_RELEASE(m_pBumpPixelConstants);
	
	SAFE_RELEASE(g_pVB_Walls);
	SAFE_RELEASE(g_pVB_Floor);	
	
	SAFE_RELEASE(g_pd3dDevice);
	SAFE_RELEASE(g_pD3D);	
}





//-----------------------------------------------------------------------------
// Name: SetupMatrices()
// Desc: Sets up the world, view, and projection transform Matrices.
//-----------------------------------------------------------------------------
VOID SetupMatrices()
{   	
	D3DXVECTOR3 vLightLookatPt( 0.5f, 0.5f, 0.5f );
    D3DXVECTOR3 vLightUpVec( 0.0f, 0.0f, 1.0f );
	//D3DXVECTOR3 vLightPos3(0.5f,1.5f,0.5f );  // set a fixed light for the shadow
	D3DXVECTOR3 vLightPos3(LightPos.x,LightPos.y,LightPos.z);	
    D3DXMatrixLookAtLH( &matLightView, &vLightPos3, &vLightLookatPt, &vLightUpVec );

	D3DXMATRIXA16 matLightProj;
    D3DXMatrixPerspectiveFovLH( &matLightProj,D3DXToRadian(110.0f), 1.0f, 0.1f, 100.0f );  
	// 110.0 deg allows to fit the shadow in the texture shadow map

    // Set up our view matrix. A view matrix can be defined given an eye point,
    // a point to lookat, and a direction for which way is up. Here, we set the
    // eye five units back along the z-axis and up three units, look at the
    // origin, and define "up" to be in the y-direction.    
    D3DXVECTOR3 vLookatPt( 0.5f, 0.5f, 0.0f );
    D3DXVECTOR3 vUpVec( 0.0f, 1.0f, 0.0f );    
    D3DXMatrixLookAtLH( &matView, &vEyePt, &vLookatPt, &vUpVec );    

    // For the projection matrix, we set up a perspective transform (which
    // transforms geometry from 3D view space to 2D viewport space, with
    // a perspective divide making objects smaller in the distance). To build
    // a perpsective transform, we need the field of view (1/4 pi is common),
    // the aspect ratio, and the near and far clipping planes (which define at
    // what distances geometry should be no longer be rendered).
    D3DXMATRIXA16 matProj;
    D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, 1.0f, 0.1f, 100.0f );    
	
	MFull = matWorld * matView * matProj;
	D3DXMatrixTranspose(&TMFull,&MFull);

	MLightFull = matWorld * matLightView * matLightProj;
	D3DXMatrixTranspose(&TMLightFull,&MLightFull);	
	
}



//-----------------------------------------------------------------------------
// Name: Draw_Walls_And_Floor()
// Desc: Draws the walls and the floor (texture & bump mapping)
//-----------------------------------------------------------------------------
void Draw_Walls_And_Floor()
{
		D3DXMatrixIdentity(&matWorld);
		// Setup the world, view, and projection Matrices
        SetupMatrices();
		int TextureIndex=0;
		D3DXHANDLE handle;
		

		g_pd3dDevice->SetVertexDeclaration(m_pBumpVertexDeclaration);
		g_pd3dDevice->SetVertexShader     (m_pBumpVertexShader);
		g_pd3dDevice->SetPixelShader      (m_pBumpPixelShader);

		

	    // set the constant parameter modelViewProj matrix for the vertrex shader
		
        if(handle = m_pBumpVertexConstants->GetConstantByName(NULL,"modelViewProj"))
			m_pBumpVertexConstants->SetMatrix(g_pd3dDevice,handle,(D3DXMATRIX*)&TMFull);
		if(handle = m_pBumpVertexConstants->GetConstantByName(NULL,"lightPos"))
			m_pBumpVertexConstants->SetVector(g_pd3dDevice,handle,(D3DXVECTOR4*)&LightPos);		

		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"lightPos"))
			m_pBumpPixelConstants->SetVector(g_pd3dDevice,handle,(D3DXVECTOR4*)&LightPos);

		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"kC"))
			m_pBumpPixelConstants->SetFloat(g_pd3dDevice,handle,kC);

		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"kL"))
			m_pBumpPixelConstants->SetFloat(g_pd3dDevice,handle,kL);

		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"kQ"))
			m_pBumpPixelConstants->SetFloat(g_pd3dDevice,handle,kQ);	
		
		kD = 1.0f;
		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"kD"))
			m_pBumpPixelConstants->SetFloat(g_pd3dDevice,handle,kD);	

		kA = 0.0f;
		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"kA"))
			m_pBumpPixelConstants->SetFloat(g_pd3dDevice,handle,kA);	

		

		if(m_pBumpPixelConstants && (handle = m_pBumpPixelConstants->GetConstantByName(NULL,"decal")))
		{
			D3DXCONSTANT_DESC constDesc;
			UINT count = 1;
			m_pBumpPixelConstants->GetConstantDesc(handle,&constDesc,&count);

			if(constDesc.RegisterSet == D3DXRS_SAMPLER)
			{
				TextureIndex = constDesc.RegisterIndex;
					
			}
			g_pd3dDevice->SetTexture( TextureIndex, g_pTexture );
			
		}
		
		

		if(m_pBumpPixelConstants && (handle = m_pBumpPixelConstants->GetConstantByName(NULL,"bump")))
			{
				
				D3DXCONSTANT_DESC constDesc;
				UINT count = 1;
				m_pBumpPixelConstants->GetConstantDesc(handle,&constDesc,&count);

				if(constDesc.RegisterSet == D3DXRS_SAMPLER)
				{
					TextureIndex = constDesc.RegisterIndex;				
					
				}
				g_pd3dDevice->SetTexture( TextureIndex, g_pWallsNormalMap );
			}				        
		
		
		// render the walls
        g_pd3dDevice->SetStreamSource( 0, g_pVB_Walls, 0, sizeof(CUSTOMVERTEX) );        
        g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 6 );
		
	

		/*-----------------------------------------------------------------------*/
		// render the floor	
		if(handle = m_pBumpVertexConstants->GetConstantByName(NULL,"modelViewProj"))
			m_pBumpVertexConstants->SetMatrix(g_pd3dDevice,handle,(D3DXMATRIX*)&TMFull);
		if(handle = m_pBumpVertexConstants->GetConstantByName(NULL,"lightPos"))
			m_pBumpVertexConstants->SetVector(g_pd3dDevice,handle,(D3DXVECTOR4*)&LightPos);		

		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"lightPos"))
			m_pBumpPixelConstants->SetVector(g_pd3dDevice,handle,(D3DXVECTOR4*)&LightPos);

		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"kC"))
			m_pBumpPixelConstants->SetFloat(g_pd3dDevice,handle,kC);

		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"kL"))
			m_pBumpPixelConstants->SetFloat(g_pd3dDevice,handle,kL);

		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"kQ"))
			m_pBumpPixelConstants->SetFloat(g_pd3dDevice,handle,kQ);	
		
		kD = 1.0f;
		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"kD"))
			m_pBumpPixelConstants->SetFloat(g_pd3dDevice,handle,kD);	

		kA = 0.0f;
		if(handle = m_pBumpPixelConstants->GetConstantByName(NULL,"kA"))
			m_pBumpPixelConstants->SetFloat(g_pd3dDevice,handle,kA);	
		

		if(m_pBumpPixelConstants && (handle = m_pBumpPixelConstants->GetConstantByName(NULL,"decal")))
		{
			D3DXCONSTANT_DESC constDesc;
			UINT count = 1;
			m_pBumpPixelConstants->GetConstantDesc(handle,&constDesc,&count);

			if(constDesc.RegisterSet == D3DXRS_SAMPLER)
			{
				TextureIndex = constDesc.RegisterIndex;
					
			}
			g_pd3dDevice->SetTexture( TextureIndex, g_pTextureFloor );
			
		}	
		

		if(m_pBumpPixelConstants && (handle = m_pBumpPixelConstants->GetConstantByName(NULL,"bump")))
			{
				
				D3DXCONSTANT_DESC constDesc;
				UINT count = 1;
				m_pBumpPixelConstants->GetConstantDesc(handle,&constDesc,&count);

				if(constDesc.RegisterSet == D3DXRS_SAMPLER)
				{
					TextureIndex = constDesc.RegisterIndex;				
					
				}
				g_pd3dDevice->SetTexture( TextureIndex, g_pFloorNormalMap );
			}				        
		
		
        g_pd3dDevice->SetStreamSource( 0, g_pVB_Floor, 0, sizeof(CUSTOMVERTEX) );        
        g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 2 );		
		
}


//-----------------------------------------------------------------------------
// Name: Draw_Particules()
// Desc: Draws the particules system
//-----------------------------------------------------------------------------
void Draw_Particules()
{
	
	g_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );
	g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
	g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_ONE );
    g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ONE );
	

	// rendu des particules
	particules->fillVertexBuffer();

	D3DXMatrixIdentity(&matWorld);
	// Setup the world, view, and projection Matrices
    SetupMatrices();	

	g_pd3dDevice->SetVertexDeclaration(m_pParticlesVertexDeclaration);
	g_pd3dDevice->SetVertexShader     (m_pParticlesVertexShader);
	g_pd3dDevice->SetPixelShader      (m_pPassPixelShader);

	D3DXHANDLE handle;
    if(handle = m_pParticlesVertexConstants->GetConstantByName(NULL,"modelViewProj"))
		m_pParticlesVertexConstants->SetMatrix(g_pd3dDevice,handle,(D3DXMATRIX*)&TMFull);

	int TextureIndex=0;
	if(m_pPassPixelConstants && (handle = m_pPassPixelConstants->GetConstantByName(NULL,"decal")))
		{
			D3DXCONSTANT_DESC constDesc;
			UINT count = 1;
			m_pPassPixelConstants->GetConstantDesc(handle,&constDesc,&count);

			if(constDesc.RegisterSet == D3DXRS_SAMPLER)
			{
				TextureIndex = constDesc.RegisterIndex;
					
			}
			g_pd3dDevice->SetTexture( TextureIndex, g_pParticuleTexture );
			
		}	

	g_pd3dDevice->SetStreamSource( 0, particules->getVertexBuffer(), 0, sizeof(PARTICLE_VERTEX) );        
    g_pd3dDevice->DrawPrimitive( D3DPT_POINTLIST, 0, particules->getNumberParticles() );	

	g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
	g_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE );
	
}


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Draws the scene
//-----------------------------------------------------------------------------
VOID Render()
{
	
    // Clear the backbuffer to a black color
    g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET| D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );
	
	ctime = timeGetTime();

    // Begin the scene
    if( SUCCEEDED( g_pd3dDevice->BeginScene() ) )
    {	
		angle+=angle_step;
		LightPos.x = (float) (center.x + rayon*cos(angle));
		LightPos.y = center.y;
		LightPos.z = (float) (center.z + rayon*sin(angle));
		LightPos.w = 1.0f;		

		Draw_Walls_And_Floor();	
		Draw_Particules();
		
		

		drawFPS(0,white);		

        // End the scene
        g_pd3dDevice->EndScene();
    }

    // Present the backbuffer contents to the display
    g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}




//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: The window's message handler
//-----------------------------------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_DESTROY:
            Cleanup();
            PostQuitMessage( 0 );
            return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}




//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: The application's entry point
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    // Register the window class
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L,
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      "HLSL basics", NULL };
    RegisterClassEx( &wc );

    // Create the application's window
    HWND hWnd = CreateWindow( "HLSL basics", "HLSL basics : texture, lighting and bump mapping",
                              WS_OVERLAPPEDWINDOW, 100, 100, 800, 600,
                              GetDesktopWindow(), NULL, wc.hInstance, NULL );
	
    // Initialize Direct3D
    if( SUCCEEDED( InitD3D( hWnd ) ) )
    {
        // Create the scene geometry
        if( SUCCEEDED( InitGeometry() ) )
        {		
			particules = new XParticles_System(g_pd3dDevice,NB_PARTICLES,SPEED_TRANSITOIRE);
			particules->Init();
            // Show the window
            ShowWindow( hWnd, SW_SHOWDEFAULT );
            UpdateWindow( hWnd );
            // Enter the message loop
            MSG msg;
            ZeroMemory( &msg, sizeof(msg) );
            while( msg.message!=WM_QUIT )
            {
                if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
                {
                    TranslateMessage( &msg );
                    DispatchMessage( &msg );
                }
                else
				{
					particules->Update();					
                    Render();
				}
            }
        }
    }

    UnregisterClass( "D3D Tutorial", wc.hInstance );
    return 0;
}



